<!--
  ~ Licensed to the Apache Software Foundation (ASF) under one
  ~ or more contributor license agreements.  See the NOTICE file
  ~ distributed with this work for additional information
  ~ regarding copyright ownership.  The ASF licenses this file
  ~ to you under the Apache License, Version 2.0 (the
  ~ "License"); you may not use this file except in compliance
  ~ with the License.  You may obtain a copy of the License at
  ~
  ~   http://www.apache.org/licenses/LICENSE-2.0
  ~
  ~ Unless required by applicable law or agreed to in writing,
  ~ software distributed under the License is distributed on an
  ~ "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  ~ KIND, either express or implied.  See the License for the
  ~ specific language governing permissions and limitations
  ~ under the License.
  -->

### Generate Delta Table for Unit Tests

To test Delta Lake ingestion, use the Python script `create_delta_table.py` to generate a sample Delta table.
Create a conda env `delta_test` with all the requirements specified in `requirements.txt` installed in the
environment:
```shell
conda create --name delta_test --file requirements.txt
```

To activate the environment:

```shell
conda activate delta_test
```

From the conda environment, you can run the python script:

```python
python3 create_delta_table.py
```

By default, the script uses `append` mode to generate 10 random records and writes the
Delta table to `resources/employee-delta-table`. You can override the defaults by supplying the command line arguments:

```shell
python3 create_delta_table.py -h

usage: create_delta_table.py [-h] [--save_mode {append,overwrite}] [--save_path SAVE_PATH] [--num_records NUM_RECORDS]

Script to write a Delta Lake table.

optional arguments:
  -h, --help            show this help message and exit
  --save_mode {append,overwrite}
                        Specify write mode (append/overwrite) (default: append)
  --save_path SAVE_PATH
                        Save path for Delta table (default: <DRUID_BASE_PATH>/druid/extensions-contrib/druid-deltalake-extensions/src/test/resources/employee-delta-table)
  --num_records NUM_RECORDS
                        Specify number of Delta records to write (default: 10)
```

The test data in `resources/employee-delta-table` was generated by:
```shell
python3 create_delta_table.py
python3 create_delta_table.py --num_records=5 --save_mode=append
```

This creates a total of 15 Delta records across two transactional commits. The resulting Delta table is checked in
to the repo. The expectated rows `DeltaTestUtils.java` are updated accordingly.
